@@ -20,6 +20,7 @@
 #define VL_LEAFIDMIN   1                 /* minimum number of leaf ID */
 #define VL_NODEIDMIN   100000000         /* minimum number of node ID */
 #define VL_VNUMBUFSIZ  8                 /* size of a buffer for variable length number */
+#define VL_NUMBUFSIZ   32                /* size of a buffer for a number */
 #define VL_LEVELMAX    64                /* max level of B+ tree */
 #define VL_DEFLRECMAX  49                /* default number of records in each leaf */
 #define VL_DEFNIDXMAX  192               /* default number of indexes in each node */
@@ -1077,6 +1078,79 @@ int vlrepair(const char *name, VLCFUNC cmp){
 }
 
 
+/* Dump all records as endian independent data. */
+int vlexportdb(VILLA *villa, const char *name){
+  DEPOT *depot;
+  char path[VL_PATHBUFSIZ], *kbuf, *vbuf, *nkey;
+  int i, err, ksiz, vsiz, ki;
+  assert(villa && name);
+  sprintf(path, "%s%s", name, VL_TMPFSUF);
+  if(!(depot = dpopen(path, DP_OWRITER | DP_OCREAT | DP_OTRUNC, -1))) return FALSE;
+  err = FALSE;
+  vlcurfirst(villa);
+  for(i = 0; !err && (kbuf = vlcurkey(villa, &ksiz)) != NULL; i++){
+    if((vbuf = vlcurval(villa, &vsiz)) != NULL){
+      nkey = cbmalloc(ksiz + VL_NUMBUFSIZ);
+      ki = sprintf(nkey, "%X\t", i);
+      memcpy(nkey + ki, kbuf, ksiz);
+      if(!dpput(depot, nkey, ki + ksiz, vbuf, vsiz, DP_DKEEP)) err = TRUE;
+      free(nkey);
+      free(vbuf);
+    } else {
+      err = TRUE;
+    }
+    free(kbuf);
+    vlcurnext(villa);
+  }
+  if(!dpexportdb(depot, name)) err = TRUE;
+  if(!dpclose(depot)) err = TRUE;
+  if(!dpremove(path)) err = TRUE;
+  return !err && !vlfatalerror(villa);
+}
+
+
+/* Load all records from endian independent data. */
+int vlimportdb(VILLA *villa, const char *name){
+  DEPOT *depot;
+  char path[VL_PATHBUFSIZ], *kbuf, *vbuf, *rp;
+  int err, ksiz, vsiz;
+  assert(villa && name);
+  if(!villa->wmode){
+    dpecodeset(DP_EMODE, __FILE__, __LINE__);
+    return FALSE;
+  }
+  if(vlrnum(villa) > 0){
+    dpecodeset(DP_EMISC, __FILE__, __LINE__);
+    return FALSE;
+  }
+  kbuf = dpname(villa->depot);
+  sprintf(path, "%s%s", kbuf, VL_TMPFSUF);
+  free(kbuf);
+  if(!(depot = dpopen(path, DP_OWRITER | DP_OCREAT | DP_OTRUNC, -1))) return FALSE;
+  err = FALSE;
+  if(!dpimportdb(depot, name)) err = TRUE;
+  dpiterinit(depot);
+  while(!err && (kbuf = dpiternext(depot, &ksiz)) != NULL){
+    if((vbuf = dpget(depot, kbuf, ksiz, 0, -1, &vsiz)) != NULL){
+      if((rp = strchr(kbuf, '\t')) != NULL){
+        rp++;
+        if(!vlput(villa, rp, ksiz - (rp - kbuf), vbuf, vsiz, VL_DDUP)) err = TRUE;
+      } else {
+        dpecodeset(DP_EBROKEN, __FILE__, __LINE__);
+        err = TRUE;
+      }
+      free(vbuf);
+    } else {
+      err = TRUE;
+    }
+    free(kbuf);
+  }
+  if(!dpclose(depot)) err = TRUE;
+  if(!dpremove(path)) err = TRUE;
+  return !err && !vlfatalerror(villa);
+}
+
+
 
 /*************************************************************************************************
  * private objects
