@@ -18,6 +18,7 @@
 
 #define CONFFILE    "rbbs.conf"          /* name of the configuration file */
 #define TEXTTYPE    "text/plain"         /* content-type of plain text */
+#define XHTMLTYPE   "application/xhtml+xml"  /* content-type of XHTML */
 #define HTMLTYPE    "text/html"          /* content-type of HTML */
 #define CSSTYPE     "text/css"           /* content-type of CSS */
 #define ATOMTYPE    "application/xml"    /* content-type of Atom */
@@ -26,9 +27,6 @@
 #define ENCODING    "UTF-8"              /* encoding of the page */
 #define PETITBNUM   31                   /* bucket number of a petit map */
 #define RDATAMAX    67108864             /* max size of data to read */
-#define PAGENAME    "rbbs.html"          /* saving filename of the page */
-#define FEEDNAME    "rbbs.xml"           /* saving filename of the feed */
-#define LATEXNAME   "rbbs.tex"           /* saving filename of the LaTeX dump */
 #define FEEDCDATE   "cdate"              /* ID for feed ordered by creation date */
 #define FEEDMDATE   "mdate"              /* ID for feed ordered by modification date */
 #define LATEXALL    "all"                /* string to get all article in LaTeX */
@@ -77,6 +75,7 @@ const char *g_pagelang = NULL;           /* language of the page */
 const char *g_pagetitle = NULL;          /* title of the page */
 const char *g_pageauthor = NULL;         /* author of the page */
 const char *g_pagesummary = NULL;        /* summary of the page */
+const char *g_keywords = NULL;           /* keywords of the site */
 int g_norobot = FALSE;                   /* whether to forbid robots */
 const char *g_captionfile = NULL;        /* path of the caption file */
 int g_memocaption = FALSE;               /* whether to show the memo in the caption */
@@ -227,7 +226,7 @@ int main(int argc, char **argv){
   CBLIST *lines;
   CBMAP *params, *cooks;
   FILE *ofp;
-  const char *tmp, *sendname;
+  const char *tmp, *sendname, *sendid;
   char *buf;
   int i, ispost;
   /* set configurations */
@@ -261,6 +260,8 @@ int main(int argc, char **argv){
       g_pageauthor = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "pagesummary:")){
       g_pagesummary = skiplabel(tmp);
+    } else if(cbstrfwimatch(tmp, "keywords:")){
+      g_keywords = skiplabel(tmp);
     } else if(cbstrfwimatch(tmp, "norobot:")){
       g_norobot = !strcmp(skiplabel(tmp), "true");
     } else if(cbstrfwimatch(tmp, "captionfile:")){
@@ -404,6 +405,7 @@ int main(int argc, char **argv){
   if(!g_pagetitle) senderror("pagetitle is undefined.");
   if(!g_pageauthor) senderror("pageauthor is undefined.");
   if(!g_pagesummary) senderror("pagesummary is undefined.");
+  if(!g_keywords) senderror("keywords is undefined.");
   if(!g_captionfile) g_captionfile = "";
   if(!g_homepageuri) senderror("homepageuri is undefined.");
   if(!g_adminpass) senderror("adminpass is undefined.");
@@ -572,42 +574,53 @@ int main(int argc, char **argv){
   free(buf);
   /* show page frame */
   sendname = "";
+  sendid = "";
   if(p_action == ACTDUMPDB){
     senddump();
     sendname = "DUMP";
   } else if(p_feed[0] != '\0'){
     sendfeed();
     sendname = "FEED";
+    sendid = p_feed;
   } else if(p_latex[0] != '\0'){
     sendlatex();
     sendname = "LATEX";
-  } else if((tmp = getenv("PATH_INFO")) != NULL){
+    sendid = p_latex;
+  } else if((tmp = getenv("PATH_INFO")) != NULL && tmp[0] != '\0'){
     if(*tmp == '/') tmp++;
     sendasis(tmp);
     sendname = "ASIS";
+    sendid = tmp;
   } else {
     sendhtml();
     sendname = "HTML";
+    sendid = p_id;
   }
+  if(sendname[0] == '\0') sendname = "*";
+  if(sendid[0] == '\0') sendid = "*";
   /* create the database if no */
   if(!g_rbbs && p_action == ACTVIEW && (g_rbbs = rbbsopen(g_dbname, TRUE)) != NULL)
     rbbsclose(g_rbbs);
   /* write the log */
   if(g_logfile[0] != '\0' && (ofp = fopen(g_logfile, "ab")) != NULL){
-    fprintf(ofp, "%s\t", g_curdate);
-    tmp = "*";
-    if(p_author[0] != '\0') tmp = p_author;
-    if(c_author[0] != '\0') tmp = c_author;
-    if(a_author[0] != '\0') tmp = a_author;
-    fprintf(ofp, "%s\t%s\t%d\t", tmp, sendname, p_action);
-    if(!(tmp = getenv("HTTP_REFERER"))) tmp = "*";
-    fprintf(ofp, "%s\t", tmp);
-    if(!(tmp = getenv("REMOTE_ADDR"))) tmp = "0.0.0.0";
-    fprintf(ofp, "%s:", tmp);
-    if(!(tmp = getenv("REMOTE_PORT"))) tmp = "0";
-    fprintf(ofp, "%s\t", tmp);
-    if(!(tmp = getenv("HTTP_USER_AGENT"))) tmp = "*";
-    fprintf(ofp, "%s\n", tmp);
+    if(fseek(ofp, 0, SEEK_SET) != -1 && lockf(fileno(ofp), F_LOCK, 0) != -1 &&
+       fseek(ofp, 0, SEEK_END) != -1){
+      fprintf(ofp, "%s\t", g_curdate);
+      tmp = "*";
+      if(p_author[0] != '\0') tmp = p_author;
+      if(c_author[0] != '\0') tmp = c_author;
+      if(a_author[0] != '\0') tmp = a_author;
+      fprintf(ofp, "%s\t%s\t%d\t%s\t", tmp, sendname, p_action, sendid);
+      if(!(tmp = getenv("HTTP_REFERER"))) tmp = "*";
+      fprintf(ofp, "%s\t", tmp);
+      if(!(tmp = getenv("REMOTE_ADDR"))) tmp = "0.0.0.0";
+      fprintf(ofp, "%s:", tmp);
+      if(!(tmp = getenv("REMOTE_PORT"))) tmp = "0";
+      fprintf(ofp, "%s\t", tmp);
+      if(!(tmp = getenv("HTTP_USER_AGENT"))) tmp = "*";
+      fprintf(ofp, "%s\n", tmp);
+      fflush(ofp);
+    }
     fclose(ofp);
   }
   return 0;
@@ -787,6 +800,7 @@ void xprintf(const char *format, ...){
 /* show the error page and exit */
 void senderror(const char *msg){
   printf("Status: 500 Internal Server Error\r\n");
+  printf("Content-Language: en\r\n");
   printf("Content-Type: %s; charset=%s\r\n", TEXTTYPE, ENCODING);
   printf("\r\n");
   printf("Error: %s\n", msg);
@@ -797,6 +811,7 @@ void senderror(const char *msg){
 /* show the error page of not found */
 void sendnotfound(void){
   printf("Status: 404 Not Found\r\n");
+  printf("Content-Language: en\r\n");
   printf("Content-Type: %s; charset=%s\r\n", TEXTTYPE, ENCODING);
   printf("\r\n");
   printf("The target of the requested URI was not found\n");
@@ -820,11 +835,13 @@ void senddump(void){
   printf("Pragma: no-cache\r\n");
   if(art){
     printf("Content-Disposition: inline; filename=%s.xml\r\n", artid(art));
+    printf("Content-Language: %s\r\n", g_pagelang);
     tmp = dateforhttp(artmdate(art), FALSE);
     printf("Last-Modified: %s\r\n", tmp);
     free(tmp);
   } else {
     printf("Content-Disposition: inline; filename=%s\r\n", DUMPNAME);
+    printf("Content-Language: %s\r\n", g_pagelang);
     tmp = dateforhttp(rbbsmdate(g_rbbs), FALSE);
     printf("Last-Modified: %s\r\n", tmp);
     free(tmp);
@@ -878,11 +895,13 @@ void sendfeed(void){
   printf("Pragma: no-cache\r\n");
   if(art){
     printf("Content-Disposition: inline; filename=%s.xml\r\n", artid(art));
+    printf("Content-Language: %s\r\n", g_pagelang);
     tmp = dateforhttp(artmdate(art), FALSE);
     printf("Last-Modified: %s\r\n", tmp);
     free(tmp);
   } else {
-    printf("Content-Disposition: inline; filename=%s\r\n", FEEDNAME);
+    printf("Content-Disposition: inline; filename=rbbs.xml\r\n");
+    printf("Content-Language: %s\r\n", g_pagelang);
     tmp = dateforhttp(rbbsmdate(g_rbbs), FALSE);
     printf("Last-Modified: %s\r\n", tmp);
     free(tmp);
@@ -894,7 +913,7 @@ void sendfeed(void){
           g_pagelang);
   if(art){
     xprintf("<link rel=\"alternate\" type=\"%@\" href=\"%@?id=%@\"/>\n",
-            HTMLTYPE, g_systemuri, artid(art));
+            XHTMLTYPE, g_systemuri, artid(art));
     xprintf("<title>%@</title>\n", artsubject(art));
     xprintf("<author>\n");
     xprintf("<name>%@</name>\n", artauthor(art));
@@ -908,7 +927,7 @@ void sendfeed(void){
     free(xml);
     freearticle(art);
   } else {
-    xprintf("<link rel=\"alternate\" type=\"%@\" href=\"%@\"/>\n", HTMLTYPE, g_systemuri);
+    xprintf("<link rel=\"alternate\" type=\"%@\" href=\"%@\"/>\n", XHTMLTYPE, g_systemuri);
     xprintf("<title>%@</title>\n", g_pagetitle);
     xprintf("<author>\n");
     xprintf("<name>%@</name>\n", g_pageauthor);
@@ -953,11 +972,13 @@ void sendlatex(void){
   printf("Pragma: no-cache\r\n");
   if(art){
     printf("Content-Disposition: inline; filename=%s.tex\r\n", artid(art));
+    printf("Content-Language: %s\r\n", g_pagelang);
     tmp = dateforhttp(artmdate(art), FALSE);
     printf("Last-Modified: %s\r\n", tmp);
     free(tmp);
   } else {
-    printf("Content-Disposition: inline; filename=%s\r\n", LATEXNAME);
+    printf("Content-Disposition: inline; filename=rbbs.tex\r\n");
+    printf("Content-Language: %s\r\n", g_pagelang);
     tmp = dateforhttp(rbbsmdate(g_rbbs), FALSE);
     printf("Last-Modified: %s\r\n", tmp);
     free(tmp);
@@ -977,6 +998,7 @@ void sendlatex(void){
     xprintf("%s\n", latexpreamble("report", g_pagetitle, g_pageauthor, rbbsmdate(g_rbbs)));
     xprintf("\\begin{document}\n\n");
     xprintf("\\maketitle\n\n");
+    xprintf("\\tableofcontents\n\n");
     while((id = getnextid(g_rbbs)) != NULL){
       if((art = getarticle(g_rbbs, id)) != NULL){
         str = arttolatex(art, TRUE);
@@ -1083,8 +1105,10 @@ const char *getmimetype(const char *name){
   if(cbstrbwimatch(name, ".xml")) return "application/xml";
   if(cbstrbwimatch(name, ".xslt")) return "application/xslt+xml";
   if(cbstrbwimatch(name, ".xhtml")) return "application/xhtml+xml";
-  if(cbstrbwimatch(name, ".atom")) return "application/atom+xml";
+  if(cbstrbwimatch(name, ".xht")) return "application/xhtml+xml";
   if(cbstrbwimatch(name, ".rdf")) return "application/rdf+xml";
+  if(cbstrbwimatch(name, ".atom")) return "application/xml";
+  if(cbstrbwimatch(name, ".rss")) return "application/xml";
   if(cbstrbwimatch(name, ".zip")) return "application/zip";
   if(cbstrbwimatch(name, ".rtf")) return "application/rtf";
   if(cbstrbwimatch(name, ".pdf")) return "application/pdf";
@@ -1103,13 +1127,16 @@ const char *getmimetype(const char *name){
 void sendhtml(void){
   const char *fstr;
   char *tmp;
+  int xhtmlok;
+  xhtmlok = (tmp = getenv("HTTP_ACCEPT")) != NULL && strstr(tmp, XHTMLTYPE);
   printf("Cache-Control: no-cache, must-revalidate\r\n");
   printf("Pragma: no-cache\r\n");
   if(p_id[0] != '\0'){
-    printf("Content-Disposition: inline; filename=%s.html\r\n", p_id);
+    printf("Content-Disposition: inline; filename=%s.%s\r\n", p_id, xhtmlok ? "xhtml" : "html");
   } else {
-    printf("Content-Disposition: inline; filename=%s\r\n", PAGENAME);
+    printf("Content-Disposition: inline; filename=rbbs.%s\r\n", xhtmlok ? "xhtml" : "html");
   }
+  printf("Content-Language: %s\r\n", g_pagelang);
   if(g_rbbs && (p_action == ACTVIEW || p_action == ACTCAL || p_action == ACTLIST ||
                 p_action == ACTSEARCH)){
     tmp = dateforhttp(rbbsmdate(g_rbbs), FALSE);
@@ -1127,7 +1154,8 @@ void sendhtml(void){
     printf("Set-Cookie: author=%s; expires=Thu, 1-Jan-2037 00:00:00 GMT\r\n", tmp);
     free(tmp);
   }
-  printf("Content-Type: %s; charset=%s\r\n", HTMLTYPE, ENCODING);
+  printf("Content-Type: %s; charset=%s\r\n", xhtmlok ? XHTMLTYPE : HTMLTYPE, ENCODING);
+  printf("Content-Style-Type: %s; charset=%s\r\n", CSSTYPE, ENCODING);
   printf("\r\n");
   xprintf("<?xml version=\"1.0\" encoding=\"%@\"?>\n", ENCODING);
   xprintf("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\""
@@ -1135,30 +1163,55 @@ void sendhtml(void){
   xprintf("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"%@\" lang=\"%@\">\n",
           g_pagelang, g_pagelang);
   xprintf("<head>\n");
-  xprintf("<title>%@</title>\n", g_pagetitle);
-  if(g_rbbs && (p_action == ACTVIEW || p_action == ACTCAL || p_action == ACTLIST ||
-                p_action == ACTSEARCH)){
-    tmp = dateforhttp(rbbsmdate(g_rbbs), FALSE);
-    xprintf("<meta http-equiv=\"Last-Modified\" content=\"%@\" />\n", tmp);
+  if(!xhtmlok){
+    xprintf("<meta http-equiv=\"Content-Language\" content=\"%@\" />\n", g_pagelang);
+    if(g_rbbs && (p_action == ACTVIEW || p_action == ACTCAL || p_action == ACTLIST ||
+                  p_action == ACTSEARCH)){
+      tmp = dateforhttp(rbbsmdate(g_rbbs), FALSE);
+      xprintf("<meta http-equiv=\"Last-Modified\" content=\"%@\" />\n", tmp);
+      free(tmp);
+    }
+    xprintf("<meta http-equiv=\"Content-Type\" content=\"%@; charset=%@\" />\n",
+            xhtmlok ? XHTMLTYPE : HTMLTYPE, ENCODING);
+    xprintf("<meta http-equiv=\"Content-Style-Type\" content=\"%@; charset=%@\" />\n",
+            CSSTYPE, ENCODING);
+  }
+  if(g_pageauthor[0] != '\0')
+    xprintf("<meta name=\"author\" content=\"%@\" />\n", g_pageauthor);
+  if(g_pagesummary[0] != '\0')
+    xprintf("<meta name=\"description\" content=\"%@\" />\n", g_pagesummary);
+  if(g_keywords[0] != '\0')
+    xprintf("<meta name=\"keywords\" content=\"%@\" />\n", g_keywords);
+  xprintf("<meta name=\"generator\" content=\"RBBS %@\" />\n", RBBS_VERSION);
+  if(g_norobot) xprintf("<meta name=\"robots\" content=\"NOINDEX, NOFOLLOW\" />\n");
+  if(g_rbbs){
+    tmp = dateforhttp(rbbscdate(g_rbbs), TRUE);
+    xprintf("<meta name=\"creation-date\" content=\"%@\" />\n", tmp);
+    free(tmp);
+    tmp = dateforhttp(rbbsmdate(g_rbbs), TRUE);
+    xprintf("<meta name=\"modification-date\" content=\"%@\" />\n", tmp);
     free(tmp);
   }
-  xprintf("<meta http-equiv=\"Content-Type\" content=\"%@; charset=%@\" />\n",
-          HTMLTYPE, ENCODING);
-  xprintf("<meta http-equiv=\"Content-Style-Type\" content=\"%@\" />\n", CSSTYPE);
-  xprintf("<meta name=\"author\" content=\"%@\" />\n", g_pageauthor);
-  xprintf("<meta name=\"generator\" content=\"RBBS %@\" />\n", RBBS_VERSION);
-  if(g_norobot) xprintf("<meta name=\"robots\" content=\"NOINDEX,NOFOLLOW\" />\n");
   tmp = dateforhttp(g_curdate, TRUE);
-  xprintf("<meta name=\"localtime\" content=\"%@\" />\n", tmp);
+  xprintf("<meta name=\"current-date\" content=\"%@\" />\n", tmp);
   free(tmp);
-  xprintf("<link rel=\"contents\" href=\"%@?action=%d\" />\n", g_systemuri, ACTLIST);
+  xprintf("<title>%@</title>\n", g_pagetitle);
+  xprintf("<link rel=\"contents\" type=\"%@\" charset=\"%@\""
+          " href=\"%@?action=%d\" title=\"List\" />\n",
+          xhtmlok ? XHTMLTYPE : HTMLTYPE, ENCODING, g_systemuri, ACTLIST);
   fstr = FEEDCDATE;
   if(g_sort == SORTMDATEA || g_sort == SORTMDATED) fstr = FEEDMDATE;
   if(p_id[0] != '\0') fstr = p_id;
-  xprintf("<link rel=\"alternate\" type=\"%@\" href=\"%@?feed=%@\" title=\"Atom\" />\n",
-          ATOMTYPE, g_systemuri, fstr);
+  xprintf("<link rel=\"alternate\" type=\"%@\" charset=\"%@\""
+          " href=\"%@?feed=%@\" title=\"Atom\" />\n",
+          ATOMTYPE, ENCODING, g_systemuri, fstr);
+  fstr = LATEXALL;
+  if(p_id[0] != '\0') fstr = p_id;
+  xprintf("<link rel=\"alternate\" type=\"%@\" charset=\"%@\""
+          " href=\"%@?latex=%@\" media=\"print\" title=\"LaTeX\" />\n",
+          LATEXTYPE, ENCODING, g_systemuri, fstr);
   if((tmp = cbreadfile(g_stylefile, NULL)) != NULL){
-    xprintf("<style type=\"%@\">\n", CSSTYPE);
+    xprintf("<style type=\"%@\">", CSSTYPE);
     xprintf("%s", tmp);
     xprintf("</style>\n");
     free(tmp);
@@ -2461,6 +2514,7 @@ void showmemo(void){
     xprintf("<input type=\"password\" name=\"userpass\" value=\"%@\" size=\"16\""
             " class=\"password\" tabindex=\"%d\" />\n", p_userpass, ++g_tabidx);
     xprintf("</td>\n");
+    xprintf("</tr>\n");
   }
   xprintf("<tr>\n");
   xprintf("<td class=\"tlabel\"></td>\n");
