@@ -364,14 +364,20 @@ char *getattribute(RBBS *rbbs, const char *name){
 
 /* Set an attribute of a database. */
 int setattribute(RBBS *rbbs, const char *name, const char *value){
+  char *rtext;
+  int err;
   assert(rbbs && name);
   if(cbstrfwmatch(name, AIDPREFIX)) return FALSE;
   if(cbstrfwmatch(name, FIDPREFIX)) return FALSE;
+  err = FALSE;
   if(value){
-    return dpput(rbbs->artdb, name, -1, value, -1, DP_DOVER);
+    rtext = strnormalize(value, FALSE, FALSE);
+    if(!(dpput(rbbs->artdb, name, -1, rtext, -1, DP_DOVER))) err = TRUE;
+    free(rtext);
   } else {
-    return dpout(rbbs->artdb, name, -1);
+    if(!dpout(rbbs->artdb, name, -1)) err = TRUE;
   }
+  return err ? FALSE : TRUE;
 }
 
 
@@ -1363,7 +1369,7 @@ char *arttoatom(ARTICLE *art, const char *systemuri, int bnum, int rnum){
   buf = cbdatumopen("", 0);
   xsprintf(buf, "<entry>\n");
   xsprintf(buf, "<id>%@?id=%@</id>\n", systemuri, art->id);
-  xsprintf(buf, "<link rel=\"alternate\" type=\"text/html\" href=\"%@?id=%@\"/>\n",
+  xsprintf(buf, "<link rel=\"alternate\" type=\"application/xhtml+xml\" href=\"%@?id=%@\"/>\n",
            systemuri, art->id);
   xsprintf(buf, "<title>%@</title>\n", art->subject);
   xsprintf(buf, "<author>\n");
@@ -1378,7 +1384,7 @@ char *arttoatom(ARTICLE *art, const char *systemuri, int bnum, int rnum){
   tmp = dateforwww(art->mdate, FALSE);
   xsprintf(buf, "<modified>%@</modified>\n", tmp);
   free(tmp);
-  xsprintf(buf, "<content type=\"text/html\" mode=\"escaped\""
+  xsprintf(buf, "<content type=\"application/xhtml+xml\" mode=\"escaped\""
            " xml:base=\"%@?id=%s\" xml:lang=\"%@\">\n", systemuri, art->id, art->language);
   tmp = bodytohtml(art, systemuri, bnum);
   xsprintf(buf, "%@", tmp);
@@ -1446,13 +1452,13 @@ char *arttolatex(ARTICLE *art, int header){
           if(elem[1] == '/'){
             xsprintf(buf, "}\n\n");
           } else {
-            xsprintf(buf, "\\section{");
+            xsprintf(buf, "\\section*{");
           }
         } else if(!strcmp(name, "subtopic")){
           if(elem[1] == '/'){
             xsprintf(buf, "}\n\n");
           } else {
-            xsprintf(buf, "\\subsection{");
+            xsprintf(buf, "\\subsection*{");
           }
         } else if(!strcmp(name, "para")){
           if(elem[1] == '/'){
@@ -1462,14 +1468,13 @@ char *arttolatex(ARTICLE *art, int header){
           if(elem[1] == '/'){
             if(!tlf) xsprintf(buf, "\n");
             xsprintf(buf, "\\end{%s}\n", vstr);
-            xsprintf(buf, "\\end{minipage}\n");
-            xsprintf(buf, "\\vspace{0.5em}\n\n");
+            xsprintf(buf, "\\end{list}\n");
+            xsprintf(buf, "\\vspace{-0.7em}\n\n");
             asis = FALSE;
             tlf = FALSE;
           } else {
-            xsprintf(buf, "\\vspace{0.5em}\n");
-            xsprintf(buf, "\\noindent \\hspace{0.8em}\n");
-            xsprintf(buf, "\\begin{minipage}[t]{\\textwidth}\n");
+            xsprintf(buf, "\\vspace{-0.7em}\n");
+            xsprintf(buf, "\\begin{list}{}{\\leftmargin=1.2em}\\item\n");
             xsprintf(buf, "\\begin{%s}\n", vstr);
             asis = TRUE;
             tlf = TRUE;
@@ -2063,6 +2068,8 @@ const char *latexpreamble(const char *dclass, const char *title,
   wp += sprintf(wp, "\\textheight=240mm\n");
   wp += sprintf(wp, "\\parindent=1.0em\n");
   wp += sprintf(wp, "\\parskip=0.7em\n");
+  if(!strcmp(dclass, "report"))
+    wp += sprintf(wp, "\\renewcommand{\\chaptername}{\\vspace{-3.5em}Article}\n");
   wp += sprintf(wp, "\\renewcommand{\\baselinestretch}{1.0}\n");
   wp += sprintf(wp, "\\let\\%s=\\verbatim\n", latexverbstr());
   wp += sprintf(wp, "\\let\\end%s=\\endverbatim\n", latexverbstr());
