@@ -675,14 +675,15 @@ int procdump(const char *name, int ftype){
       xprintf("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"%@\" lang=\"%@\">\n",
               g_dumplang, g_dumplang);
       xprintf("<head>\n");
-      xprintf("<title>%@</title>\n", g_dumptitle);
       tmp = dateforhttp(rbbsmdate(rbbs), FALSE);
       xprintf("<meta http-equiv=\"Last-Modified\" content=\"%@\" />\n", tmp);
       free(tmp);
       xprintf("<meta http-equiv=\"Content-Type\" content=\"%@; charset=%@\" />\n",
               HTMLTYPE, ENCODING);
       xprintf("<meta name=\"author\" content=\"%@\" />\n", g_dumpauthor);
+      xprintf("<meta name=\"description\" content=\"%@\" />\n", g_dumpsummary);
       xprintf("<meta name=\"generator\" content=\"RBBS %@\" />\n", RBBS_VERSION);
+      xprintf("<title>%@</title>\n", g_dumptitle);
       xprintf("<link rel=\"contents\" href=\"%@?action=2\" />\n", g_systemuri);
       xprintf("</head>\n");
       xprintf("<body>\n");
@@ -709,6 +710,7 @@ int procdump(const char *name, int ftype){
       printf("%s\n", latexpreamble("report", g_dumptitle, g_dumpauthor, rbbsmdate(rbbs)));
       printf("\\begin{document}\n\n");
       printf("\\maketitle\n\n");
+      printf("\\tableofcontents\n\n");
       break;
     default:
       xprintf("<?xml version=\"1.0\" encoding=\"%@\"?>\n", ENCODING);
